@@ -1,45 +1,67 @@
+# -*- coding: utf-8 -*-
 """
 State tracking functionality for django models
 """
-
-from __future__ import annotations
-
 import inspect
-from functools import partialmethod
+import sys
 from functools import wraps
 
-from django.apps import apps as django_apps
 from django.db import models
-from django.db.models import Field
-from django.db.models.query_utils import DeferredAttribute
 from django.db.models.signals import class_prepared
+from django.utils.functional import curry
+from django_fsm.signals import pre_transition, post_transition
 
-from django_fsm.signals import post_transition
-from django_fsm.signals import pre_transition
+try:
+    from django.apps import apps as django_apps
 
-__all__ = [
-    "TransitionNotAllowed",
-    "ConcurrentTransition",
-    "FSMFieldMixin",
-    "FSMField",
-    "FSMIntegerField",
-    "FSMKeyField",
-    "ConcurrentTransitionMixin",
-    "transition",
-    "can_proceed",
-    "has_transition_perm",
-    "GET_STATE",
-    "RETURN_VALUE",
-]
+    def get_model(app_label, model_name):
+        app = django_apps.get_app_config(app_label)
+        return app.get_model(model_name)
+except ImportError:
+    from django.db.models.loading import get_model
+
+
+__all__ = ['TransitionNotAllowed', 'ConcurrentTransition',
+           'FSMFieldMixin', 'FSMField', 'FSMIntegerField',
+           'FSMKeyField', 'ConcurrentTransitionMixin', 'transition',
+           'can_proceed', 'has_transition_perm']
+
+if sys.version_info[:2] == (2, 6):
+    # Backport of Python 2.7 inspect.getmembers,
+    # since Python 2.6 ships buggy implementation
+    def __getmembers(object, predicate=None):
+        """Return all members of an object as (name, value) pairs sorted by name.
+        Optionally, only return members that satisfy a given predicate."""
+        results = []
+        for key in dir(object):
+            try:
+                value = getattr(object, key)
+            except AttributeError:
+                continue
+            if not predicate or predicate(value):
+                results.append((key, value))
+        results.sort()
+        return results
+    inspect.getmembers = __getmembers
+
+# South support; see http://south.aeracode.org/docs/tutorial/part4.html#simple-inheritance
+try:
+    from south.modelsinspector import add_introspection_rules
+except ImportError:
+    pass
+else:
+    add_introspection_rules([], [r"^django_fsm\.FSMField"])
+    add_introspection_rules([], [r"^django_fsm\.FSMIntegerField"])
+    add_introspection_rules([], [r"^django_fsm\.FSMKeyField"])
 
 
 class TransitionNotAllowed(Exception):
     """Raised when a transition is not allowed"""
 
     def __init__(self, *args, **kwargs):
-        self.object = kwargs.pop("object", None)
-        self.method = kwargs.pop("method", None)
-        super().__init__(*args, **kwargs)
+        self.object = kwargs.pop('object', None)
+        self.method = kwargs.pop('method', None)
+        super(TransitionNotAllowed, self).__init__(*args, **kwargs)
 
 
 class InvalidResultState(Exception):
@@ -54,7 +76,7 @@ class ConcurrentTransition(Exception):
     """
 
 
-class Transition:
+class Transition(object):
     def __init__(self, method, source, target, on_error, conditions, permission, custom):
         self.method = method
         self.source = source
@@ -71,24 +93,14 @@ class Transition:
     def has_perm(self, instance, user):
         if not self.permission:
             return True
-        if callable(self.permission):
+        elif callable(self.permission):
             return bool(self.permission(instance, user))
-        if user.has_perm(self.permission, instance):
+        elif user.has_perm(self.permission, instance):
             return True
-        if user.has_perm(self.permission):
+        elif user.has_perm(self.permission):
             return True
-        return False
-
-    def __hash__(self):
-        return hash(self.name)
-
-    def __eq__(self, other):
-        if isinstance(other, str):
-            return other == self.name
-        if isinstance(other, Transition):
-            return other.name == self.name
-
-        return False
+        else:
+            return False
 
 
 def get_available_FIELD_transitions(instance, field):
@@ -99,7 +111,7 @@ def get_available_FIELD_transitions(instance, field):
     curr_state = field.get_state(instance)
     transitions = field.transitions[instance.__class__]
 
-    for transition in transitions.values():
+    for name, transition in transitions.items():
         meta = transition._django_fsm
         if meta.has_transition(curr_state) and meta.conditions_met(instance, curr_state):
             yield meta.get_transition(curr_state)
@@ -122,11 +134,10 @@ def get_available_user_FIELD_transitions(instance, user, field):
             yield transition
 
 
-class FSMMeta:
+class FSMMeta(object):
     """
     Models methods transitions meta information
     """
-
     def __init__(self, field, method):
         self.field = field
         self.transitions = {}  # source -> Transition
@@ -134,14 +145,14 @@ class FSMMeta:
     def get_transition(self, source):
         transition = self.transitions.get(source, None)
         if transition is None:
-            transition = self.transitions.get("*", None)
+            transition = self.transitions.get('*', None)
         if transition is None:
-            transition = self.transitions.get("+", None)
+            transition = self.transitions.get('+', None)
         return transition
 
     def add_transition(self, method, source, target, on_error=None, conditions=[], permission=None, custom={}):
         if source in self.transitions:
-            raise AssertionError(f"Duplicate transition for {source} state")
+            raise AssertionError('Duplicate transition for {0} state'.format(source))
 
         self.transitions[source] = Transition(
             method=method,
@@ -150,8 +161,7 @@ class FSMMeta:
             on_error=on_error,
             conditions=conditions,
             permission=permission,
-            custom=custom,
-        )
+            custom=custom)
 
     def has_transition(self, state):
         """
@@ -160,10 +170,10 @@ class FSMMeta:
         if state in self.transitions:
             return True
 
-        if "*" in self.transitions:
+        if '*' in self.transitions:
             return True
 
-        if "+" in self.transitions and self.transitions["+"].target != state:
+        if '+' in self.transitions and self.transitions['+'].target != state:
             return True
 
         return False
@@ -176,25 +186,24 @@ class FSMMeta:
 
         if transition is None:
             return False
-
-        if transition.conditions is None:
+        elif transition.conditions is None:
             return True
-
-        return all(condition(instance) for condition in transition.conditions)
+        else:
+            return all(map(lambda condition: condition(instance), transition.conditions))
 
     def has_transition_perm(self, instance, state, user):
         transition = self.get_transition(state)
 
         if not transition:
             return False
-
-        return transition.has_perm(instance, user)
+        else:
+            return transition.has_perm(instance, user)
 
     def next_state(self, current_state):
         transition = self.get_transition(current_state)
 
         if transition is None:
-            raise TransitionNotAllowed(f"No transition from {current_state}")
+            raise TransitionNotAllowed('No transition from {0}'.format(current_state))
 
         return transition.target
 
@@ -202,12 +211,12 @@ class FSMMeta:
         transition = self.get_transition(current_state)
 
         if transition is None:
-            raise TransitionNotAllowed(f"No transition from {current_state}")
+            raise TransitionNotAllowed('No transition from {0}'.format(current_state))
 
         return transition.on_error
 
 
-class FSMFieldDescriptor:
+class FSMFieldDescriptor(object):
     def __init__(self, field):
         self.field = field
 
@@ -218,45 +227,43 @@ class FSMFieldDescriptor:
 
     def __set__(self, instance, value):
         if self.field.protected and self.field.name in instance.__dict__:
-            raise AttributeError(f"Direct {self.field.name} modification is not allowed")
+            raise AttributeError('Direct {0} modification is not allowed'.format(self.field.name))
 
         # Update state
         self.field.set_proxy(instance, value)
         self.field.set_state(instance, value)
 
 
-class FSMFieldMixin:
+class FSMFieldMixin(object):
     descriptor_class = FSMFieldDescriptor
 
     def __init__(self, *args, **kwargs):
-        self.protected = kwargs.pop("protected", False)
+        self.protected = kwargs.pop('protected', False)
         self.transitions = {}  # cls -> (transitions name -> method)
         self.state_proxy = {}  # state -> ProxyClsRef
 
-        state_choices = kwargs.pop("state_choices", None)
-        choices = kwargs.get("choices", None)
+        state_choices = kwargs.pop('state_choices', None)
+        choices = kwargs.get('choices', None)
         if state_choices is not None and choices is not None:
-            raise ValueError("Use one of choices or state_choices value")
+            raise ValueError('Use one of choices or state_choices value')
 
         if state_choices is not None:
             choices = []
             for state, title, proxy_cls_ref in state_choices:
                 choices.append((state, title))
                 self.state_proxy[state] = proxy_cls_ref
-            kwargs["choices"] = choices
+            kwargs['choices'] = choices
 
-        super().__init__(*args, **kwargs)
+        super(FSMFieldMixin, self).__init__(*args, **kwargs)
 
     def deconstruct(self):
-        name, path, args, kwargs = super().deconstruct()
+        name, path, args, kwargs = super(FSMFieldMixin, self).deconstruct()
         if self.protected:
-            kwargs["protected"] = self.protected
+            kwargs['protected'] = self.protected
         return name, path, args, kwargs
 
     def get_state(self, instance):
-        # The state field may be deferred. We delegate the logic of figuring this out
-        # and loading the deferred field on-demand to Django's built-in DeferredAttribute class.
-        return DeferredAttribute(self).__get__(instance)
+        return instance.__dict__[self.name]
 
     def set_state(self, instance, state):
         instance.__dict__[self.name] = state
@@ -275,10 +282,9 @@ class FSMFieldMixin:
                 app_label = instance._meta.app_label
                 model_name = state_proxy
 
-            model = django_apps.get_app_config(app_label).get_model(model_name)
-
+            model = get_model(app_label, model_name)
             if model is None:
-                raise ValueError(f"No model found {state_proxy}")
+                raise ValueError('No model found {0}'.format(state_proxy))
 
             instance.__class__ = model
 
@@ -289,26 +295,24 @@ class FSMFieldMixin:
 
         if not meta.has_transition(current_state):
             raise TransitionNotAllowed(
-                f"Can't switch from state '{current_state}' using method '{method_name}'",
-                object=instance,
-                method=method,
-            )
+                "Can't switch from state '{0}' using method '{1}'".format(current_state, method_name),
+                object=instance, method=method)
         if not meta.conditions_met(instance, current_state):
             raise TransitionNotAllowed(
-                f"Transition conditions have not been met for method '{method_name}'", object=instance, method=method
-            )
+                "Transition conditions have not been met for method '{0}'".format(method_name),
+                object=instance, method=method)
 
         next_state = meta.next_state(current_state)
 
         signal_kwargs = {
-            "sender": instance.__class__,
-            "instance": instance,
-            "name": method_name,
-            "field": meta.field,
-            "source": current_state,
-            "target": next_state,
-            "method_args": args,
-            "method_kwargs": kwargs,
+            'sender': instance.__class__,
+            'instance': instance,
+            'name': method_name,
+            'field': meta.field,
+            'source': current_state,
+            'target': next_state,
+            'method_args' : args,
+            'method_kwargs' : kwargs
         }
 
         pre_transition.send(**signal_kwargs)
@@ -316,9 +320,11 @@ class FSMFieldMixin:
         try:
             result = method(instance, *args, **kwargs)
             if next_state is not None:
-                if hasattr(next_state, "get_state"):
-                    next_state = next_state.get_state(instance, transition, result, args=args, kwargs=kwargs)
-                    signal_kwargs["target"] = next_state
+                if hasattr(next_state, 'get_state'):
+                    next_state = next_state.get_state(
+                        instance, transition, result,
+                        args=args, kwargs=kwargs)
+                    signal_kwargs['target'] = next_state
                 self.set_proxy(instance, next_state)
                 self.set_state(instance, next_state)
         except Exception as exc:
@@ -326,8 +332,8 @@ class FSMFieldMixin:
             if exception_state:
                 self.set_proxy(instance, exception_state)
                 self.set_state(instance, exception_state)
-                signal_kwargs["target"] = exception_state
-                signal_kwargs["exception"] = exc
+                signal_kwargs['target'] = exception_state
+                signal_kwargs['exception'] = exc
                 post_transition.send(**signal_kwargs)
             raise
         else:
@@ -341,7 +347,7 @@ class FSMFieldMixin:
         """
         transitions = self.transitions[instance_cls]
 
-        for transition in transitions.values():
+        for name, transition in transitions.items():
             meta = transition._django_fsm
 
             for transition in meta.transitions.values():
@@ -350,37 +356,27 @@ class FSMFieldMixin:
     def contribute_to_class(self, cls, name, **kwargs):
         self.base_cls = cls
 
-        super().contribute_to_class(cls, name, **kwargs)
+        super(FSMFieldMixin, self).contribute_to_class(cls, name, **kwargs)
         setattr(cls, self.name, self.descriptor_class(self))
-        setattr(cls, f"get_all_{self.name}_transitions", partialmethod(get_all_FIELD_transitions, field=self))
-        setattr(cls, f"get_available_{self.name}_transitions", partialmethod(get_available_FIELD_transitions, field=self))
-        setattr(
-            cls,
-            f"get_available_user_{self.name}_transitions",
-            partialmethod(get_available_user_FIELD_transitions, field=self),
-        )
+        setattr(cls, 'get_all_{0}_transitions'.format(self.name),
+                curry(get_all_FIELD_transitions, field=self))
+        setattr(cls, 'get_available_{0}_transitions'.format(self.name),
+                curry(get_available_FIELD_transitions, field=self))
+        setattr(cls, 'get_available_user_{0}_transitions'.format(self.name),
+                curry(get_available_user_FIELD_transitions, field=self))
 
         class_prepared.connect(self._collect_transitions)
 
     def _collect_transitions(self, *args, **kwargs):
-        sender = kwargs["sender"]
+        sender = kwargs['sender']
 
         if not issubclass(sender, self.base_cls):
             return
 
         def is_field_transition_method(attr):
-            return (
-                (inspect.ismethod(attr) or inspect.isfunction(attr))
-                and hasattr(attr, "_django_fsm")
-                and (
-                    attr._django_fsm.field in [self, self.name]
-                    or (
-                        isinstance(attr._django_fsm.field, Field)
-                        and attr._django_fsm.field.name == self.name
-                        and attr._django_fsm.field.creation_counter == self.creation_counter
-                    )
-                )
-            )
+            return (inspect.ismethod(attr) or inspect.isfunction(attr)) \
+                and hasattr(attr, '_django_fsm') \
+                and attr._django_fsm.field in [self, self.name]
 
         sender_transitions = {}
         transitions = inspect.getmembers(sender, predicate=is_field_transition_method)
@@ -395,17 +391,15 @@ class FSMField(FSMFieldMixin, models.CharField):
     """
     State Machine support for Django model as CharField
     """
-
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault("max_length", 50)
-        super().__init__(*args, **kwargs)
+        kwargs.setdefault('max_length', 50)
+        super(FSMField, self).__init__(*args, **kwargs)
 
 
 class FSMIntegerField(FSMFieldMixin, models.IntegerField):
     """
     Same as FSMField, but stores the state value in an IntegerField.
     """
-
     pass
 
 
@@ -413,7 +407,6 @@ class FSMKeyField(FSMFieldMixin, models.ForeignKey):
     """
     State Machine support for Django model
     """
-
     def get_state(self, instance):
         return instance.__dict__[self.attname]
 
@@ -421,34 +414,7 @@ class FSMKeyField(FSMFieldMixin, models.ForeignKey):
         instance.__dict__[self.attname] = self.to_python(state)
 
 
-class FSMModelMixin:
-    """
-    Mixin that allows refresh_from_db for models with fsm protected fields
-    """
-
-    def _get_protected_fsm_fields(self):
-        def is_fsm_and_protected(f):
-            return isinstance(f, FSMFieldMixin) and f.protected
-
-        protected_fields = filter(is_fsm_and_protected, self._meta.concrete_fields)
-        return {f.attname for f in protected_fields}
-
-    def refresh_from_db(self, *args, **kwargs):
-        fields = kwargs.pop("fields", None)
-
-        # Use provided fields, if not set then reload all non-deferred fields.0
-        if not fields:
-            deferred_fields = self.get_deferred_fields()
-            protected_fields = self._get_protected_fsm_fields()
-            skipped_fields = deferred_fields.union(protected_fields)
-
-            fields = [f.attname for f in self._meta.concrete_fields if f.attname not in skipped_fields]
-
-        kwargs["fields"] = fields
-        super().refresh_from_db(*args, **kwargs)
-
-
-class ConcurrentTransitionMixin:
+class ConcurrentTransitionMixin(object):
     """
     Protects a Model from undesirable effects caused by concurrently executed transitions,
     e.g. running the same transition multiple times at the same time, or running different
@@ -473,14 +439,16 @@ class ConcurrentTransitionMixin:
     a rollback of all the changes that have been executed in an inconsistent (out of sync)
     state, thus practically negating their effect.
     """
-
     def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+        super(ConcurrentTransitionMixin, self).__init__(*args, **kwargs)
         self._update_initial_state()
 
     @property
     def state_fields(self):
-        return filter(lambda field: isinstance(field, FSMFieldMixin), self._meta.fields)
+        return filter(
+            lambda field: isinstance(field, FSMFieldMixin),
+            self._meta.fields
+        )
 
     def _do_update(self, base_qs, using, pk_val, values, update_fields, forced_update):
         # _do_update is called once for each model class in the inheritance hierarchy.
@@ -490,15 +458,15 @@ class ConcurrentTransitionMixin:
         filter_on = filter(lambda field: field.model == base_qs.model, self.state_fields)
 
         # state filter will be used to narrow down the standard filter checking only PK
-        state_filter = {field.attname: self.__initial_states[field.attname] for field in filter_on}
+        state_filter = dict((field.attname, self.__initial_states[field.attname]) for field in filter_on)
 
-        updated = super()._do_update(
+        updated = super(ConcurrentTransitionMixin, self)._do_update(
             base_qs=base_qs.filter(**state_filter),
             using=using,
             pk_val=pk_val,
             values=values,
             update_fields=update_fields,
-            forced_update=forced_update,
+            forced_update=forced_update
         )
 
         # It may happen that nothing was updated in the original _do_update method not because of unmatching state,
@@ -507,37 +475,34 @@ class ConcurrentTransitionMixin:
         # INSERT if UPDATE fails.
         # Thus, we need to make sure we only catch the case when the object *is* in the DB, but with changed state; and
         # mimic standard _do_update behavior otherwise. Django will pick it up and execute _do_insert.
-        if not updated and base_qs.filter(pk=pk_val).using(using).exists():
+        if not updated and base_qs.filter(pk=pk_val).exists():
             raise ConcurrentTransition("Cannot save object! The state has been changed since fetched from the database!")
 
         return updated
 
     def _update_initial_state(self):
-        self.__initial_states = {field.attname: field.value_from_object(self) for field in self.state_fields}
-
-    def refresh_from_db(self, *args, **kwargs):
-        super().refresh_from_db(*args, **kwargs)
-        self._update_initial_state()
+        self.__initial_states = dict(
+            (field.attname, field.value_from_object(self)) for field in self.state_fields
+        )
 
     def save(self, *args, **kwargs):
-        super().save(*args, **kwargs)
+        super(ConcurrentTransitionMixin, self).save(*args, **kwargs)
         self._update_initial_state()
 
 
-def transition(field, source="*", target=None, on_error=None, conditions=[], permission=None, custom={}):
+def transition(field, source='*', target=None, on_error=None, conditions=[], permission=None, custom={}):
     """
-    Method decorator to mark allowed transitions.
+    Method decorator for mark allowed transitions
 
     Set target to None if current state needs to be validated and
-    has not changed after the function call.
+    has not changed after the function call
     """
-
     def inner_transition(func):
-        wrapper_installed, fsm_meta = True, getattr(func, "_django_fsm", None)
+        wrapper_installed, fsm_meta = True, getattr(func, '_django_fsm', None)
         if not fsm_meta:
             wrapper_installed = False
             fsm_meta = FSMMeta(field=field, method=func)
-            setattr(func, "_django_fsm", fsm_meta)
+            setattr(func, '_django_fsm', fsm_meta)
 
         if isinstance(source, (list, tuple, set)):
             for state in source:
@@ -564,35 +529,36 @@ def can_proceed(bound_method, check_conditions=True):
     Set ``check_conditions`` argument to ``False`` to skip checking
     conditions.
     """
-    if not hasattr(bound_method, "_django_fsm"):
-        raise TypeError(f"{bound_method.__func__.__name__} method is not transition")
+    if not hasattr(bound_method, '_django_fsm'):
+        im_func = getattr(bound_method, 'im_func', getattr(bound_method, '__func__'))
+        raise TypeError('%s method is not transition' % im_func.__name__)
 
     meta = bound_method._django_fsm
-    self = bound_method.__self__
-    current_state = meta.field.get_state(self)
+    im_self = getattr(bound_method, 'im_self', getattr(bound_method, '__self__'))
+    current_state = meta.field.get_state(im_self)
 
-    return meta.has_transition(current_state) and (not check_conditions or meta.conditions_met(self, current_state))
+    return meta.has_transition(current_state) and (
+        not check_conditions or meta.conditions_met(im_self, current_state))
 
 
 def has_transition_perm(bound_method, user):
     """
     Returns True if model in state allows to call bound_method and user have rights on it
     """
-    if not hasattr(bound_method, "_django_fsm"):
-        raise TypeError(f"{bound_method.__func__.__name__} method is not transition")
+    if not hasattr(bound_method, '_django_fsm'):
+        im_func = getattr(bound_method, 'im_func', getattr(bound_method, '__func__'))
+        raise TypeError('%s method is not transition' % im_func.__name__)
 
     meta = bound_method._django_fsm
-    self = bound_method.__self__
-    current_state = meta.field.get_state(self)
+    im_self = getattr(bound_method, 'im_self', getattr(bound_method, '__self__'))
+    current_state = meta.field.get_state(im_self)
 
-    return (
-        meta.has_transition(current_state)
-        and meta.conditions_met(self, current_state)
-        and meta.has_transition_perm(self, current_state, user)
-    )
+    return (meta.has_transition(current_state) and
+            meta.conditions_met(im_self, current_state) and
+            meta.has_transition_perm(im_self, current_state, user))
 
 
-class State:
+class State(object):
     def get_state(self, model, transition, result, args=[], kwargs={}):
         raise NotImplementedError
 
@@ -604,7 +570,9 @@ class RETURN_VALUE(State):
     def get_state(self, model, transition, result, args=[], kwargs={}):
         if self.allowed_states is not None:
             if result not in self.allowed_states:
-                raise InvalidResultState(f"{result} is not in list of allowed states\n{self.allowed_states}")
+                raise InvalidResultState(
+                    '{} is not in list of allowed states\n{}'.format(
+                        result, self.allowed_states))
         return result
 
 
@@ -617,5 +585,7 @@ class GET_STATE(State):
         result_state = self.func(model, *args, **kwargs)
         if self.allowed_states is not None:
             if result_state not in self.allowed_states:
-                raise InvalidResultState(f"{result_state} is not in list of allowed states\n{self.allowed_states}")
+                raise InvalidResultState(
+                    '{} is not in list of allowed states\n{}'.format(
+                        result, self.allowed_states))
         return result_state
