@@ -1,12 +1,7 @@
-from __future__ import annotations
-
 from django.db import models
 from django.test import TestCase
-
-from django_fsm import FSMField
-from django_fsm import TransitionNotAllowed
-from django_fsm import can_proceed
-from django_fsm import transition
+from django_fsm import FSMField, TransitionNotAllowed, \
+    transition, can_proceed
 
 
 def condition_func(instance):
@@ -14,7 +9,7 @@ def condition_func(instance):
 
 
 class BlogPostWithConditions(models.Model):
-    state = FSMField(default="new")
+    state = FSMField(default='new')
 
     def model_condition(self):
         return True
@@ -22,11 +17,13 @@ class BlogPostWithConditions(models.Model):
     def unmet_condition(self):
         return False
 
-    @transition(field=state, source="new", target="published", conditions=[condition_func, model_condition])
+    @transition(field=state, source='new', target='published',
+                conditions=[condition_func, model_condition])
     def publish(self):
         pass
 
-    @transition(field=state, source="published", target="destroyed", conditions=[condition_func, unmet_condition])
+    @transition(field=state, source='published', target='destroyed',
+                conditions=[condition_func, unmet_condition])
     def destroy(self):
         pass
 
@@ -36,17 +33,18 @@ class ConditionalTest(TestCase):
         self.model = BlogPostWithConditions()
 
     def test_initial_staet(self):
-        self.assertEqual(self.model.state, "new")
+        self.assertEqual(self.model.state, 'new')
 
     def test_known_transition_should_succeed(self):
         self.assertTrue(can_proceed(self.model.publish))
         self.model.publish()
-        self.assertEqual(self.model.state, "published")
+        self.assertEqual(self.model.state, 'published')
 
     def test_unmet_condition(self):
         self.model.publish()
-        self.assertEqual(self.model.state, "published")
+        self.assertEqual(self.model.state, 'published')
         self.assertFalse(can_proceed(self.model.destroy))
         self.assertRaises(TransitionNotAllowed, self.model.destroy)
 
-        self.assertTrue(can_proceed(self.model.destroy, check_conditions=False))
+        self.assertTrue(can_proceed(self.model.destroy,
+                                    check_conditions=False))
