@@ -1,14 +1,9 @@
-from __future__ import annotations
-
 from django.db import models
 from django.test import TestCase
-
-from django_fsm import FSMIntegerField
-from django_fsm import TransitionNotAllowed
-from django_fsm import transition
+from django_fsm import FSMIntegerField, TransitionNotAllowed, transition
 
 
-class BlogPostStateEnum:
+class BlogPostStateEnum(object):
     NEW = 10
     PUBLISHED = 20
     HIDDEN = 30
@@ -39,3 +34,6 @@ class BlogPostWithIntegerFieldTest(TestCase):
 
     def test_unknow_transition_fails(self):
         self.assertRaises(TransitionNotAllowed, self.model.hide)
+
+
+
