@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 try:
     from django.contrib.contenttypes.fields import GenericForeignKey
 except ImportError:
@@ -8,24 +6,23 @@ except ImportError:
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
 from django.test import TestCase
-
-from django_fsm import FSMField
-from django_fsm import transition
+from django_fsm import FSMField, transition
 
 
 class Ticket(models.Model):
+
     class Meta:
-        app_label = "testapp"
+        app_label = 'testapp'
 
 
 class Task(models.Model):
     class STATE:
-        NEW = "new"
-        DONE = "done"
+        NEW = 'new'
+        DONE = 'done'
 
-    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
+    content_type = models.ForeignKey(ContentType)
     object_id = models.PositiveIntegerField()
-    causality = GenericForeignKey("content_type", "object_id")
+    causality = GenericForeignKey('content_type', 'object_id')
     state = FSMField(default=STATE.NEW)
 
     @transition(field=state, source=STATE.NEW, target=STATE.DONE)
@@ -33,7 +30,7 @@ class Task(models.Model):
         pass
 
     class Meta:
-        app_label = "testapp"
+        app_label = 'testapp'
 
 
 class Test(TestCase):
